@@ -15,8 +15,10 @@
 The library root. See :mod:`~trieste.bayesian_optimizer` for the core optimizer, which requires
 models (see :mod:`~trieste.models`), and data sets (see :mod:`~trieste.data`). The
 :mod:`~trieste.acquisition` package provides a selection of acquisition algorithms and the
-functionality to define your own. The :mod:`~trieste.objectives` package contains several popular
-objective functions, useful for experimentation.
+functionality to define your own. The :mod:`~trieste.ask_tell_optimization` package provides API
+for Ask-Tell optimization and manual control of the optimization loop.
+The :mod:`~trieste.objectives` package contains several popular objective functions,
+useful for experimentation.
 
 Bibliography
 ============
@@ -25,12 +27,13 @@ Bibliography
 """
 from . import (
     acquisition,
+    ask_tell_optimization,
     bayesian_optimizer,
     data,
     models,
     objectives,
     observer,
     space,
-    type,
+    types,
     utils,
 )
