@@ -57,7 +57,8 @@ from .function import (
     augmented_expected_improvement,
     expected_hv_improvement,
     expected_improvement,
-    gibbon,
+    gibbon_quality_term,
+    gibbon_repulsion_term,
     hard_local_penalizer,
     lower_confidence_bound,
     min_value_entropy_search,
