@@ -25,35 +25,41 @@ from typing import Generic, Optional, TypeVar, Union
 
 import tensorflow as tf
 
+from .. import types
 from ..data import Dataset
 from ..models import ProbabilisticModel
 from ..observer import OBJECTIVE
 from ..space import Box, SearchSpace
-from ..type import TensorType
+from ..types import TensorType
 from .function import (
+    AcquisitionFunction,
     AcquisitionFunctionBuilder,
     ExpectedImprovement,
     GreedyAcquisitionFunctionBuilder,
     SingleModelAcquisitionBuilder,
     SingleModelGreedyAcquisitionBuilder,
 )
-from .optimizer import (
-    AcquisitionOptimizer,
-    automatic_optimizer_selector,
-    batchify,
-    generate_continuous_optimizer,
-)
+from .optimizer import AcquisitionOptimizer, automatic_optimizer_selector, batchify
 from .sampler import ExactThompsonSampler, RandomFourierFeatureThompsonSampler, ThompsonSampler
 
-S = TypeVar("S")
-""" Unbound type variable. """
+T_co = TypeVar("T_co", covariant=True)
+""" Unbound covariant type variable. """
 
 SP_contra = TypeVar("SP_contra", bound=SearchSpace, contravariant=True)
 """ Contravariant type variable bound to :class:`~trieste.space.SearchSpace`. """
 
 
-class AcquisitionRule(ABC, Generic[S, SP_contra]):
-    """The central component of the acquisition API."""
+class AcquisitionRule(ABC, Generic[T_co, SP_contra]):
+    """
+    The central component of the acquisition API.
+
+    An :class:`AcquisitionRule` can produce any value from the search space for this step, and the
+    historic data and models. This value is typically a set of query points, either on its own as
+    a `TensorType` (see e.g. :class:`EfficientGlobalOptimization`), or within some context
+    (see e.g. :class:`TrustRegion`). Indeed, to use an :class:`AcquisitionRule` in the main
+    :class:`~trieste.bayesian_optimizer.BayesianOptimizer` Bayesian optimization loop, the rule
+    must return either a `TensorType` or `State`-ful `TensorType`.
+    """
 
     @abstractmethod
     def acquire(
@@ -61,30 +67,21 @@ class AcquisitionRule(ABC, Generic[S, SP_contra]):
         search_space: SP_contra,
         datasets: Mapping[str, Dataset],
         models: Mapping[str, ProbabilisticModel],
-        state: S | None = None,
-    ) -> tuple[TensorType, S]:
+    ) -> T_co:
         """
-        Return the optimal points within the specified ``search_space``, where optimality is defined
-        by the acquisition rule.
-
+        Return a value of type `T_co`. Typically this will be a set of query points, either on its
+        own as a `TensorType` (see e.g. :class:`EfficientGlobalOptimization`), or within some
+        context (see e.g. :class:`TrustRegion`).
 
         **Type hints:**
-          - The global search space must be a :class:`~trieste.space.SearchSpace`. The exact type
-            of :class:`~trieste.space.SearchSpace` depends on the specific
-            :class:`AcquisitionRule`.
-          - Each :class:`AcquisitionRule` must define the type of its corresponding acquisition
-            state (if the rule is stateless, this type can be `None`). The ``state`` passed
-            to this method, and the state returned, must both be of that type.
+          - The search space must be a :class:`~trieste.space.SearchSpace`. The exact type of
+            :class:`~trieste.space.SearchSpace` depends on the specific :class:`AcquisitionRule`.
 
-
-        :param search_space: The global search space over which the optimization problem
-            is defined.
+        :param search_space: The local acquisition search space for *this step*.
         :param datasets: The known observer query points and observations for each tag.
         :param models: The model to use for each :class:`~trieste.data.Dataset` in ``datasets``
             (matched by tag).
-        :param state: The acquisition state from the previous step, if there was a previous step,
-            else `None`.
-        :return: The optimal points and the acquisition state for this step.
+        :return: A value of type `T_co`.
         """
 
     def acquire_single(
@@ -92,31 +89,25 @@ class AcquisitionRule(ABC, Generic[S, SP_contra]):
         search_space: SP_contra,
         dataset: Dataset,
         model: ProbabilisticModel,
-        state: S | None = None,
-    ) -> tuple[TensorType, S]:
+    ) -> T_co:
         """
         A convenience wrapper for :meth:`acquire` that uses only one model, dataset pair.
 
-        Return the optimal points within the specified ``search_space``, where optimality is defined
-        by the acquisition rule.
-
         :param search_space: The global search space over which the optimization problem
             is defined.
         :param dataset: The known observer query points and observations.
-        :param models: The model to use for the dataset.
-        :param state: The acquisition state from the previous step, if there was a previous step,
-            else `None`.
-        :return: The optimal points and the acquisition state for this step.
+        :param model: The model to use for the dataset.
+        :return: A value of type `T_co`.
         """
         if isinstance(dataset, dict) or isinstance(model, dict):
             raise ValueError(
                 "AcquisitionRule.acquire_single method does not support multiple datasets "
                 "or models: use acquire instead"
             )
-        return self.acquire(search_space, {OBJECTIVE: dataset}, {OBJECTIVE: model}, state)
+        return self.acquire(search_space, {OBJECTIVE: dataset}, {OBJECTIVE: model})
 
 
-class EfficientGlobalOptimization(AcquisitionRule[None, SP_contra]):
+class EfficientGlobalOptimization(AcquisitionRule[TensorType, SP_contra]):
     """Implements the Efficient Global Optimization, or EGO, algorithm."""
 
     def __init__(
@@ -169,6 +160,7 @@ class EfficientGlobalOptimization(AcquisitionRule[None, SP_contra]):
         self._builder: Union[AcquisitionFunctionBuilder, GreedyAcquisitionFunctionBuilder] = builder
         self._optimizer = optimizer
         self._num_query_points = num_query_points
+        self._acquisition_function: Optional[AcquisitionFunction] = None
 
     def __repr__(self) -> str:
         """"""
@@ -182,37 +174,41 @@ class EfficientGlobalOptimization(AcquisitionRule[None, SP_contra]):
         search_space: SP_contra,
         datasets: Mapping[str, Dataset],
         models: Mapping[str, ProbabilisticModel],
-        state: None = None,
-    ) -> tuple[TensorType, None]:
+    ) -> TensorType:
         """
-        Return the query point that optimizes the acquisition function produced by ``builder`` (see
-        :meth:`__init__`).
+        Return the query point(s) that optimizes the acquisition function produced by ``builder``
+        (see :meth:`__init__`).
 
-        :param search_space: The global :class:`~trieste.space.SearchSpace` over which the
-            optimization problem is defined.
+        :param search_space: The local acquisition search space for *this step*.
         :param datasets: The known observer query points and observations.
         :param models: The models of the specified ``datasets``.
-        :param state: Unused.
-        :return: The single (or batch of) points to query, and `None`.
+        :return: The single (or batch of) points to query.
         """
+        if self._acquisition_function is None:
+            self._acquisition_function = self._builder.prepare_acquisition_function(
+                datasets, models
+            )
+        else:
+            self._acquisition_function = self._builder.update_acquisition_function(
+                self._acquisition_function, datasets, models
+            )
 
-        acquisition_function = self._builder.prepare_acquisition_function(datasets, models)
-        points = self._optimizer(search_space, acquisition_function)
+        points = self._optimizer(search_space, self._acquisition_function)
 
         if isinstance(self._builder, GreedyAcquisitionFunctionBuilder):
             for _ in range(
                 self._num_query_points - 1
             ):  # greedily allocate remaining batch elements
-                greedy_acquisition_function = self._builder.prepare_acquisition_function(
-                    datasets, models, pending_points=points
+                self._acquisition_function = self._builder.update_acquisition_function(
+                    self._acquisition_function, datasets, models, pending_points=points
                 )
-                chosen_point = self._optimizer(search_space, greedy_acquisition_function)
+                chosen_point = self._optimizer(search_space, self._acquisition_function)
                 points = tf.concat([points, chosen_point], axis=0)
 
-        return points, None
+        return points
 
 
-class DiscreteThompsonSampling(AcquisitionRule[None, SearchSpace]):
+class DiscreteThompsonSampling(AcquisitionRule[TensorType, SearchSpace]):
     r"""
     Implements Thompson sampling for choosing optimal points.
 
@@ -267,19 +263,16 @@ class DiscreteThompsonSampling(AcquisitionRule[None, SearchSpace]):
         search_space: SearchSpace,
         datasets: Mapping[str, Dataset],
         models: Mapping[str, ProbabilisticModel],
-        state: None = None,
-    ) -> tuple[TensorType, None]:
+    ) -> TensorType:
         """
         Sample `num_search_space_samples` (see :meth:`__init__`) points from the
         ``search_space``. Of those points, return the `num_query_points` points at which
         random samples yield the **minima** of the model posterior.
 
-        :param search_space: The global :class:`~trieste.space.SearchSpace` over which the
-            optimization problem is defined.
+        :param search_space: The local acquisition search space for *this step*.
         :param datasets: Unused.
         :param models: The model of the known data. Uses the single key `OBJECTIVE`.
-        :param state: Unused.
-        :return: The `num_query_points` points to query, and `None`.
+        :return: The ``num_query_points`` points to query.
         :raise ValueError: If ``models`` do not contain the key `OBJECTIVE`, or it contains any
             other key.
         """
@@ -311,10 +304,10 @@ class DiscreteThompsonSampling(AcquisitionRule[None, SearchSpace]):
         query_points = search_space.sample(self._num_search_space_samples)
         thompson_samples = thompson_sampler.sample(query_points)
 
-        return thompson_samples, None
+        return thompson_samples
 
 
-class TrustRegion(AcquisitionRule["TrustRegion.State", Box]):
+class TrustRegion(AcquisitionRule[types.State[Optional["TrustRegion.State"], TensorType], Box]):
     """Implements the *trust region* acquisition algorithm."""
 
     @dataclass(frozen=True)
@@ -344,52 +337,43 @@ class TrustRegion(AcquisitionRule["TrustRegion.State", Box]):
 
     def __init__(
         self,
-        builder: Optional[AcquisitionFunctionBuilder | SingleModelAcquisitionBuilder] = None,
+        rule: AcquisitionRule[TensorType, Box] | None = None,
         beta: float = 0.7,
         kappa: float = 1e-4,
-        optimizer: AcquisitionOptimizer[Box] | None = None,
     ):
         """
-        :param builder: The acquisition function builder to use. :class:`TrustRegion` will attempt
-            to **maximise** the corresponding acquisition function. Defaults to
-            :class:`~trieste.acquisition.ExpectedImprovement` with tag `OBJECTIVE`.
+        :param rule: The acquisition rule that defines how to search for a new query point in a
+            given search space. Defaults to :class:`EfficientGlobalOptimization` with default
+            arguments.
         :param beta: The inverse of the trust region contraction factor.
         :param kappa: Scales the threshold for the minimal improvement required for a step to be
             considered a success.
-        :param optimizer: The optimizer with which to optimize the acquisition function built by
-            ``builder``. This must be able optimize over a :class:`Box`.
         """
-        if builder is None:
-            builder = ExpectedImprovement()
+        if rule is None:
+            rule = EfficientGlobalOptimization()
 
-        if isinstance(builder, SingleModelAcquisitionBuilder):
-            builder = builder.using(OBJECTIVE)
-
-        if optimizer is None:
-            optimizer = generate_continuous_optimizer()
-
-        self._builder = builder
+        self._rule = rule
         self._beta = beta
         self._kappa = kappa
-        self._optimizer = optimizer
 
     def __repr__(self) -> str:
         """"""
-        return f"TrustRegion({self._builder!r}, {self._beta!r}, {self._kappa!r})"
+        return f"TrustRegion({self._rule!r}, {self._beta!r}, {self._kappa!r})"
 
     def acquire(
         self,
         search_space: Box,
         datasets: Mapping[str, Dataset],
         models: Mapping[str, ProbabilisticModel],
-        state: State | None = None,
-    ) -> tuple[TensorType, State]:
+    ) -> types.State[State | None, TensorType]:
         """
-        Acquire one new query point according the trust region algorithm. Return the new query point
-        along with the final acquisition state from this step.
+        Construct a local search space from ``search_space`` according the trust region algorithm,
+        and use that with the ``rule`` specified at :meth:`~TrustRegion.__init__` to find new
+        query points. Return a function that constructs these points given a previous trust region
+        state.
 
-        If no ``state`` is specified (it is `None`), ``search_space`` is used as
-        the search space for this step.
+        If no ``state`` is specified (it is `None`), ``search_space`` is used as the search space
+        for this step.
 
         If a ``state`` is specified, and the new optimum improves over the previous optimum
         by some threshold (that scales linearly with ``kappa``), the previous acquisition is
@@ -408,14 +392,12 @@ class TrustRegion(AcquisitionRule["TrustRegion.State", Box]):
         ``search_space``. For a local search, the actual search space will be the
         intersection of the trust region and ``search_space``.
 
-        :param search_space: The global  :class:`~trieste.space.SearchSpace` for the optimization
-            problem.
+        :param search_space: The local acquisition search space for *this step*.
         :param datasets: The known observer query points and observations. Uses the data for key
             `OBJECTIVE` to calculate the new trust region.
         :param models: The models of the specified ``datasets``.
-        :param state: The acquisition state from the previous step, if there was a previous step,
-            else `None`.
-        :return: A 2-tuple of the query point and the acquisition state for this step.
+        :return: A function that constructs the next acquisition state and the recommended query
+            points from the previous acquisition state.
         :raise KeyError: If ``datasets`` does not contain the key `OBJECTIVE`.
         """
         dataset = datasets[OBJECTIVE]
@@ -425,36 +407,39 @@ class TrustRegion(AcquisitionRule["TrustRegion.State", Box]):
 
         y_min = tf.reduce_min(dataset.observations, axis=0)
 
-        if state is None:
-            eps = 0.5 * (global_upper - global_lower) / (5.0 ** (1.0 / global_lower.shape[-1]))
-            is_global = True
-        else:
-            tr_volume = tf.reduce_prod(
-                state.acquisition_space.upper - state.acquisition_space.lower
-            )
-            step_is_success = y_min < state.y_min - self._kappa * tr_volume
+        def go(state: TrustRegion.State | None) -> tuple[TrustRegion.State | None, TensorType]:
 
-            eps = (
-                state.eps
-                if state.is_global
-                else state.eps / self._beta
-                if step_is_success
-                else state.eps * self._beta
-            )
+            if state is None:
+                eps = 0.5 * (global_upper - global_lower) / (5.0 ** (1.0 / global_lower.shape[-1]))
+                is_global = True
+            else:
+                tr_volume = tf.reduce_prod(
+                    state.acquisition_space.upper - state.acquisition_space.lower
+                )
+                step_is_success = y_min < state.y_min - self._kappa * tr_volume
 
-            is_global = step_is_success or not state.is_global
+                eps = (
+                    state.eps
+                    if state.is_global
+                    else state.eps / self._beta
+                    if step_is_success
+                    else state.eps * self._beta
+                )
 
-        if is_global:
-            acquisition_space = search_space
-        else:
-            xmin = dataset.query_points[tf.argmin(dataset.observations)[0], :]
-            acquisition_space = Box(
-                tf.reduce_max([global_lower, xmin - eps], axis=0),
-                tf.reduce_min([global_upper, xmin + eps], axis=0),
-            )
+                is_global = step_is_success or not state.is_global
 
-        acquisition_function = self._builder.prepare_acquisition_function(datasets, models)
-        point = self._optimizer(acquisition_space, acquisition_function)
-        state_ = TrustRegion.State(acquisition_space, eps, y_min, is_global)
+            if is_global:
+                acquisition_space = search_space
+            else:
+                xmin = dataset.query_points[tf.argmin(dataset.observations)[0], :]
+                acquisition_space = Box(
+                    tf.reduce_max([global_lower, xmin - eps], axis=0),
+                    tf.reduce_min([global_upper, xmin + eps], axis=0),
+                )
 
-        return point, state_
+            points = self._rule.acquire(acquisition_space, datasets, models)
+            state_ = TrustRegion.State(acquisition_space, eps, y_min, is_global)
+
+            return state_, points
+
+        return go
