@@ -28,7 +28,7 @@ from scipy.optimize import bisect
 
 from ..data import Dataset
 from ..models import ProbabilisticModel
-from ..type import TensorType
+from ..types import TensorType
 from ..utils import DEFAULTS
 
 
@@ -237,7 +237,7 @@ class IndependentReparametrizationSampler(Sampler):
 
         if tf.size(self._eps) == 0:
             self._eps.assign(
-                tf.random.normal([self._sample_size, mean.shape[-1]], dtype=tf.float64)
+                tf.random.normal([self._sample_size, tf.shape(mean)[-1]], dtype=tf.float64)
             )  # [S, L]
 
         return mean + tf.sqrt(var) * tf.cast(self._eps[:, None, :], var.dtype)  # [..., S, 1, L]
@@ -269,6 +269,10 @@ class BatchReparametrizationSampler(Sampler):
             tf.ones([0, 0, sample_size], dtype=tf.float64), shape=[None, None, sample_size]
         )  # [0, 0, S]
 
+        # for some reason graph compilation is resulting in self._eps reporting the wrong shape
+        # we therefore use an extra boolean variable to keep track of whether it's initialised
+        self._initialized = tf.Variable(False)
+
     def sample(self, at: TensorType, *, jitter: float = DEFAULTS.JITTER) -> TensorType:
         """
         Return approximate samples from the `model` specified at :meth:`__init__`. Multiple calls to
@@ -298,9 +302,7 @@ class BatchReparametrizationSampler(Sampler):
 
         tf.debugging.assert_positive(batch_size)
 
-        eps_is_populated = tf.size(self._eps) != 0
-
-        if eps_is_populated:
+        if self._initialized:
             tf.debugging.assert_equal(
                 batch_size,
                 tf.shape(self._eps)[-2],
@@ -310,12 +312,13 @@ class BatchReparametrizationSampler(Sampler):
 
         mean, cov = self._model.predict_joint(at)  # [..., B, L], [..., L, B, B]
 
-        if not eps_is_populated:
+        if not self._initialized:
             self._eps.assign(
                 tf.random.normal(
-                    [mean.shape[-1], batch_size, self._sample_size], dtype=tf.float64
+                    [tf.shape(mean)[-1], batch_size, self._sample_size], dtype=tf.float64
                 )  # [L, B, S]
             )
+            self._initialized.assign(True)
 
         identity = tf.eye(batch_size, dtype=cov.dtype)  # [B, B]
         cov_cholesky = tf.linalg.cholesky(cov + jitter * identity)  # [..., L, B, B]
@@ -412,7 +415,7 @@ class RandomFourierFeatureThompsonSampler(ThompsonSampler):
 
         try:
             self._noise_variance = model.get_observation_noise()
-            self._kernel = model.get_kernel()  # type: ignore
+            self._kernel = model.get_kernel()
         except (NotImplementedError, AttributeError):
             raise ValueError(
                 """
@@ -439,23 +442,19 @@ class RandomFourierFeatureThompsonSampler(ThompsonSampler):
         return f"""{self.__class__.__name__}(
         {self._sample_size!r},
         {self._model!r},
-        {self._sample_min_value},
+        {self._sample_min_value!r},
         {self._num_features!r})
         """
 
     def _prepare_theta_posterior_in_design_space(self) -> tfp.distributions.Distribution:
-        r"""
-        Calculate the posterior of theta (the feature weights) in the design space. This
-        distribution is a Gaussian
-
-        .. math:: \theta \sim N(D^{-1}\Phi^Ty,D^{-1}\sigma^2)
-
-        where the [m,m] design matrix :math:`D=(\Phi^T\Phi + \sigma^2I_m)` is defined for
-        the [n,m] matrix of feature evaluations across the training data :math:`\Phi`
-        and observation noise variance :math:`\sigma^2`.
-
-        :return: The posterior distribution for theta.
-        """
+        # Calculate the posterior of theta (the feature weights) in the design space. This
+        # distribution is a Gaussian
+        #
+        # .. math:: \theta \sim N(D^{-1}\Phi^Ty,D^{-1}\sigma^2)
+        #
+        # where the [m,m] design matrix :math:`D=(\Phi^T\Phi + \sigma^2I_m)` is defined for
+        # the [n,m] matrix of feature evaluations across the training data :math:`\Phi`
+        # and observation noise variance :math:`\sigma^2`.
 
         phi = self._feature_functions(self._dataset.query_points)  # [n, m]
         D = tf.matmul(phi, phi, transpose_a=True)  # [m, m]
@@ -475,17 +474,13 @@ class RandomFourierFeatureThompsonSampler(ThompsonSampler):
         )
 
     def _prepare_theta_posterior_in_gram_space(self) -> tfp.distributions.Distribution:
-        r"""
-        Calculate the posterior of theta (the feature weights) in the gram space.
-
-         .. math:: \theta \sim N(\Phi^TG^{-1}y,I_m - \Phi^TG^{-1}\Phi)
-
-        where the [n,n] gram matrix :math:`G=(\Phi\Phi^T + \sigma^2I_n)` is defined for the [n,m]
-        matrix of feature evaluations across the training data :math:`\Phi` and
-        observation noise variance :math:`\sigma^2`.
-
-        :return: The posterior distribution for theta.
-        """
+        # Calculate the posterior of theta (the feature weights) in the gram space.
+        #
+        #  .. math:: \theta \sim N(\Phi^TG^{-1}y,I_m - \Phi^TG^{-1}\Phi)
+        #
+        # where the [n,n] gram matrix :math:`G=(\Phi\Phi^T + \sigma^2I_n)` is defined for the [n,m]
+        # matrix of feature evaluations across the training data :math:`\Phi` and
+        # observation noise variance :math:`\sigma^2`.
 
         phi = self._feature_functions(self._dataset.query_points)  # [n, m]
         G = tf.matmul(phi, phi, transpose_b=True)  # [n, n]
