@@ -1,4 +1,4 @@
-# Copyright 2020 The Trieste Contributors
+# Copyright 2021 The Trieste Contributors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,25 +11,28 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+
 from __future__ import annotations
 
+from collections.abc import Callable
 from dataclasses import dataclass, field
-from typing import Any, Dict, Union
+from typing import Any, Union
 
-import gpflow
 import tensorflow as tf
 
-from .model_interfaces import TrainableProbabilisticModel, supported_models
-from .optimizer import create_optimizer
-
-
-def _default_optimizer() -> gpflow.optimizers.Scipy:
-    return gpflow.optimizers.Scipy()
+from .interfaces import TrainableProbabilisticModel
+from .optimizer import Optimizer, create_optimizer
 
 
 @dataclass(frozen=True)
 class ModelConfig:
-    """Specification for building a :class:`~trieste.models.TrainableProbabilisticModel`."""
+    """
+    This class is a specification for building a
+    :class:`~trieste.models.TrainableProbabilisticModel`. It is not meant to be used by itself,
+    subclasses that implement the missing asbtract methods should be used instead. These abstract
+    methods define a default optimizer and all models supported by a specific model type (e.g.
+    Gaussian processes implementation). Note that subclasses should also be frozen dataclasses.
+    """
 
     model: tf.Module | TrainableProbabilisticModel
     """ The :class:`~trieste.models.TrainableProbabilisticModel`, or the model to wrap in one. """
@@ -37,9 +40,7 @@ class ModelConfig:
     model_args: dict[str, Any] = field(default_factory=lambda: {})
     """ The keyword arguments to pass to the model wrapper. """
 
-    optimizer: gpflow.optimizers.Scipy | tf.optimizers.Optimizer = field(
-        default_factory=_default_optimizer
-    )
+    optimizer: Any = field(default_factory=lambda: tf.optimizers.Adam())
     """ The optimizer with which to train the model (by minimizing its loss function). """
 
     optimizer_args: dict[str, Any] = field(default_factory=lambda: {})
@@ -48,21 +49,23 @@ class ModelConfig:
     def __post_init__(self) -> None:
         self._check_model_type()
 
-    @staticmethod
-    def create_from_dict(d: dict[str, Any]) -> ModelConfig:
+    def supported_models(
+        self,
+    ) -> dict[Any, Callable[[Any, Optimizer], TrainableProbabilisticModel]]:
         """
-        :param d: A dictionary from which to construct this :class:`ModelConfig`.
-        :return: A :class:`ModelConfig` built from ``d``.
-        :raise TypeError: If the keys in ``d`` do not correspond to the parameters of
-            :class:`ModelConfig`.
+        Defines all models supported by certain model type (e.g. Gaussian process implementation).
+        This method has to be specified by a model type specific subclass.
+
+        :return: A mapping of third-party model types to :class:`CustomTrainable` classes that wrap
+            models of those types.
         """
-        return ModelConfig(**d)
+        raise NotImplementedError
 
     def _check_model_type(self) -> None:
         if isinstance(self.model, TrainableProbabilisticModel):
             return
 
-        for model_type in supported_models:
+        for model_type in self.supported_models():
             if isinstance(self.model, model_type):
                 return
 
@@ -77,14 +80,14 @@ class ModelConfig:
 
         optimizer = create_optimizer(self.optimizer, self.optimizer_args)
 
-        for model_type, model_interface in supported_models.items():
+        for model_type, model_interface in self.supported_models().items():
             if isinstance(self.model, model_type):
                 return model_interface(self.model, optimizer, **self.model_args)  # type: ignore
 
         raise NotImplementedError(f"Not supported type {type(self.model)}")
 
 
-ModelSpec = Union[Dict[str, Any], ModelConfig, TrainableProbabilisticModel]
+ModelSpec = Union[ModelConfig, TrainableProbabilisticModel]
 """ Type alias for any type that can be used to fully specify a model. """
 
 
@@ -95,8 +98,6 @@ def create_model(config: ModelSpec) -> TrainableProbabilisticModel:
     """
     if isinstance(config, ModelConfig):
         return config.create_model_interface()
-    elif isinstance(config, dict):
-        return ModelConfig(**config).create_model_interface()
     elif isinstance(config, TrainableProbabilisticModel):
         return config
     raise NotImplementedError("Unknown format passed to create a TrainableProbabilisticModel.")
