@@ -14,9 +14,6 @@
 """ This package contains library utilities. """
 from typing import Callable, Mapping, TypeVar
 
-from typing_extensions import Final
-
-from . import pareto
 from .misc import DEFAULTS, Err, Ok, Result, T_co, jit, shapes_equal, to_numpy
 
 K = TypeVar("K")
